"""AES cipher implementation"""

import base64
import os

from cryptography.exceptions import InvalidTag
from cryptography.hazmat.primitives.ciphers.aead import AESGCM as _AESGCM

from .base_cipher import BaseCipher, FailToDecryptData, InvalidKeyProvided

NONCE_LENGTH = 96


class AESGCM(BaseCipher):
    """
    AES-GCM cipher implementation

    Attributes:
        key (bytes or str): encryption key in bytes or base64 encoded
        nonceLen (int): length of nonce generated per each encryption
        aesgcm (_AESGCM): aes cipher
    """

    auxiliaryDataLen = NONCE_LENGTH + 16

    def __init__(self, key: bytes | str):
        if isinstance(key, str):
            try:
                key = base64.b64decode(key)
            except Exception as exc:
                raise InvalidKeyProvided("Bad base64 string") from exc
        self.key = key
        try:
            self.aesgcm = _AESGCM(self.key)
        except ValueError as err:
            raise InvalidKeyProvided("Invalid key length. Key must be 256 bits") from err

    def encrypt(self, data: bytes) -> bytes:
        """
        Encrypt data

        Args:
            data: data to encrypt
        Returns:
            encrypted data in format encrypted_data + tag(16) + nonce(96)
        """
        nonce = os.urandom(NONCE_LENGTH)
        return self.aesgcm.encrypt(nonce, data, None) + nonce

    def decrypt(self, data: bytes) -> bytes:
        """
        Decrypt data

        Args:
            data: data to decrypt
        Returns:
            decrypted data
        Raises:
            FailToDecryptData: for unsuccessful decryption attempt
        """
        try:
            return self.aesgcm.decrypt(data[-NONCE_LENGTH:], data[:-NONCE_LENGTH], None)
        except InvalidTag as exc:
            raise FailToDecryptData() from exc
